@@ -1,3 +1,12 @@
+/*
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
+*/
 /*
  * Copyright (c) 1989, 1993
  *	The Regents of the University of California.  All rights reserved.
@@ -52,28 +61,10 @@ static char sccsid[] = "@(#)net.c	8.4 (Berkeley) 4/28/95";
 #include <string.h>
 #include "finger.h"
 
-#if FASTCTO
-#include <signal.h>
-#include <setjmp.h>
-#endif /* FASTCTO */
-
-#if IPSEC
-#include <errno.h>
-#endif /* IPSEC */
-
 #if INET6
 #include "support.h"
 #endif /* INET6 */
 
-#ifdef FASTCTO
-static jmp_buf timeout_env;
-
-static void timeout_handler(int i)
-{
-  longjmp(timeout_env, i);
-}
-#endif /* FASTCTO */
-
 void
 netfinger(name)
 	char *name;
@@ -108,66 +99,20 @@ netfinger(name)
 	req.ai_flags |= AI_CANONNAME;
 
 	if (s = getaddrinfo(host, "finger", &req, &ai)) {
-          fprintf(stderr, "finger: getaddrinfo: %s: %s\n", host, nrl_gai_strerror(s));
+          fprintf(stderr, "finger: getaddrinfo: %s: %s\n", host, gai_strerror(s));
 	  return;
 	}
 
-#ifdef FASTCTO
-	signal(SIGALRM, &timeout_handler);
-#endif /* FASTCTO */
-
-	{
-	  for (ai2 = ai, s = -1; ai; ai = ai->ai_next) {
-	    if ((s = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
-	      perror("socket");
-	      continue;
-	    }
-
-#if IPSEC
-            if (request)
-              if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
-                fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
-                exit(1);
-              };
-#endif /* IPSEC */
-
-#if FASTCTO
-	    if (setjmp(timeout_env)) {
-	      fprintf(stderr, "finger: Connection timed out\n");
-	      continue;
-	    }
-
-	    alarm(FASTCTO);
-#endif /* FASTCTO */
-
-	    if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, buf1, sizeof(buf1), buf2, sizeof(buf2), 0)) {
-	      printf("finger: getnameinfo() failed!\n");
-	      continue;
-	    }
-
-	    printf("Trying %s.%s...\n", buf1, buf2);
-	    
-	    if (connect(s, ai->ai_addr, ai->ai_addrlen) < 0) {
-#ifdef FASTCTO
-	      alarm(0);
-#endif /* FASTCTO */
-	      perror("connect");
-	      close(s);
-	      s = -1;
-	      continue;
-	    }
-#ifdef FASTCTO
-	    alarm(0);
-#endif /* FASTCTO */
-	    break;
-	  }
-
-	  if (s < 0)
-	    return;
-	}
-	printf("[%s]\n", ai->ai_canonname);
-
-	freeaddrinfo(ai2);
+#if NETSEC
+        s = inner_connect(ai, request, requestlen, NULL, NULL, "finger", &ai2);
+#else /* NETSEC */
+        s = inner_connect(ai, NULL, 0, NULL, NULL, "finger", &ai2);
+#endif /* NETSEC */
+	if (s >= 0)
+          printf("[%s]\n", ai2->ai_canonname);
+	freeaddrinfo(ai);
+        if (s < 0)
+          return;
 #else /* INET6 */
 	if (!(host = rindex(name, '@')))
 		return;
