@@ -1,3 +1,12 @@
+/*
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
+*/
 /*
  * Copyright (c) 1985, 1989, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -67,10 +76,6 @@ static char sccsid[] = "@(#)ftp.c	8.6 (Berkeley) 10/27/94";
 
 #include "ftp_var.h"
 
-#if FASTCTO
-#include <setjmp.h>
-#endif /* FASTCTO */
-
 #if INET6
 #include "support.h"
 #endif /* INET6 */
@@ -103,23 +108,14 @@ off_t	restart_point = 0;
 
 FILE	*cin, *cout;
 
-#if linux
+#if defined(__linux__) && !defined(__GLIBC__)
 typedef void (*sig_t)(int);
-#endif /* linux */
+#endif /* defined(__linux__) && !defined(__GLIBC__) */
 
-#if IPSEC
+#if NETSEC
 extern void *control_request, *data_request;
 extern int control_requestlen, data_requestlen;
-#endif /* IPSEC */
-
-#ifdef FASTCTO
-static jmp_buf timeout_env;
-
-static void timeout_handler(int i)
-{
-  longjmp(timeout_env, i);
-}
-#endif /* FASTCTO */
+#endif /* NETSEC */
 
 char *
 hookup(host, port)
@@ -146,73 +142,23 @@ hookup(host, port)
     port = "ftp";
 
   if (i = getaddrinfo(host, port, &req, &ai)) {
-    fprintf(stderr, "ftp: getaddrinfo: %s.%s: %s\n", host, port, nrl_gai_strerror(i));
+    fprintf(stderr, "ftp: getaddrinfo: %s.%s: %s\n", host, port, gai_strerror(i));
     return NULL;
   }
 
-#ifdef FASTCTO
-  signal(SIGALRM, &timeout_handler);
-#endif /* FASTCTO */
-  
-  for (ai2 = ai, s = -1; ai; ai = ai->ai_next) {
-    if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, buf1, sizeof(buf1), buf2, sizeof(buf2), 0)) {
-      printf("ftp: getnameinfo() failed!\n");
-      continue;
-    }
-    
-    printf("Trying %s.%s...\n", buf1, buf2);
-    
-    if (s >= 0)
-      close(s);
-    
-    if ((s = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
-      perror("ftp: socket");
-      continue;
-    }
-    
-#if IPSEC
-    if (control_request)
-      if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, control_request, control_requestlen) < 0)
-        fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, control_request, ...): %s(%d)\n", strerror(errno), errno);
-#endif /* IPSEC */
+#if NETSEC
+  s = inner_connect(ai, control_request, control_requestlen, NULL, NULL, "ftp", &ai2);
+#else /* NETSEC */
+  s = inner_connect(ai, NULL, 0, NULL, NULL, "ftp", &ai2);
+#endif /* NETSEC */
+  if (s >= 0)
+    strcpy(hostname = hostnamebuf, ai2->ai_canonname);
+  freeaddrinfo(ai);
 
-#if FASTCTO
-    if (setjmp(timeout_env)) {
-      fprintf(stderr, "ftp: Connection timed out\n");
-      continue;
-    }
+  if (s < 0)
+    return NULL;
 
-    alarm(FASTCTO);
-#endif /* FASTCTO */
-
-    if (connect(s, ai->ai_addr, ai->ai_addrlen) < 0) {
-#ifdef FASTCTO
-      alarm(0);
-#endif /* FASTCTO */
-      perror("ftp: connect");
-      errno = 0;
-      close(s);
-      s = -1;
-      continue;
-    }
-
-#ifdef FASTCTO
-      alarm(0);
-#endif /* FASTCTO */
-      memcpy(&hisctladdr, ai->ai_addr, ai->ai_addrlen);
-      strcpy(hostname = hostnamebuf, ai->ai_canonname);
-    }
-
-#ifdef FASTCTO
-    signal(SIGALRM, SIG_IGN);
-#endif /* FASTCTO */
-
-    freeaddrinfo(ai2);
-
-    if (s < 0)
-      return NULL;
-
-    howport = 0;
+  howport = 0;
 #else /* INET6 */
 	struct hostent *hp = 0;
 	int s, len, tos;
@@ -548,14 +494,14 @@ getreply(expecteof)
 
 int
 empty(mask, sec)
-	struct fd_set *mask;
+	fd_set *mask;
 	int sec;
 {
 	struct timeval t;
 
 	t.tv_sec = (long) sec;
 	t.tv_usec = 0;
-	return (select(32, mask, (struct fd_set *) 0, (struct fd_set *) 0, &t));
+	return (select(32, mask, (fd_set *) 0, (fd_set *) 0, &t));
 }
 
 jmp_buf	sendabort;
@@ -1156,11 +1102,11 @@ initconn()
 		    setsockopt(data, SOL_SOCKET, SO_DEBUG, (char *)&on,
 			       sizeof (on)) < 0)
 			perror("ftp: setsockopt (ignored)");
-#if IPSEC
+#if NETSEC
                 if (data_request)
 		  if (setsockopt(data, SOL_SOCKET, SO_SECURITY_REQUEST, data_request, data_requestlen) < 0)
 		    fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, data_request, ...): %s(%d)\n", strerror(errno), errno);
-#endif /* IPSEC */
+#endif /* NETSEC */
 #if INET6
 		switch (howport) {
 		  case 0:
@@ -1182,6 +1128,7 @@ initconn()
 		    }
 		    if (howport)
 		      goto bad;
+#if !INNER
 		  case 3:
 		    result = command("LPSV");
 		    if ((result == COMPLETE) && !lporttosa(&data_addr, pasv, &hisctladdr)) {
@@ -1190,6 +1137,7 @@ initconn()
 		    }
 		    if (howport)
 		      goto bad;
+#endif /* !INNER */
 		  default:
 		    goto bad;
 		}
@@ -1283,11 +1231,11 @@ noport:
 			sendport = 1;
 		return (1);
 	}
-#if IPSEC
+#if NETSEC
         if (data_request)
 	  if (setsockopt(data, SOL_SOCKET, SO_SECURITY_REQUEST, data_request, data_requestlen) < 0)
 	    fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, data_request, ...): %s(%d)\n", strerror(errno), errno);
-#endif /* IPSEC */
+#endif /* NETSEC */
 	if (!sendport)
 		if (setsockopt(data, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof (on)) < 0) {
 			warn("setsockopt (reuse address)");
@@ -1353,6 +1301,7 @@ noport:
 	      }
 	      if (howport)
 		return -1;
+#if !INNER
 	    case 3:
 	      if (satolport(buffer, &data_addr)) {
 		printf("Can't convert address for transmission\n");
@@ -1365,6 +1314,7 @@ noport:
 	      }
 	      if (howport)
 		return -1;
+#endif /* !INNER */
 	    default:
 	      return 1;
 	  }
@@ -1616,7 +1566,7 @@ proxtrans(cmd, local, remote)
 	sig_t oldintr;
 	int secndflag = 0, prox_type, nfnd;
 	char *cmd2;
-	struct fd_set mask;
+	fd_set mask;
 
 	if (strcmp(cmd, "RETR"))
 		cmd2 = "RETR";
@@ -1733,7 +1683,7 @@ reset(argc, argv)
 	int argc;
 	char *argv[];
 {
-	struct fd_set mask;
+	fd_set mask;
 	int nfnd = 1;
 
 	FD_ZERO(&mask);
@@ -1802,7 +1752,7 @@ abort_remote(din)
 {
 	char buf[BUFSIZ];
 	int nfnd;
-	struct fd_set mask;
+	fd_set mask;
 
 	/*
 	 * send IAC in urgent mode instead of DM because 4.3BSD places oob mark
