@@ -1,3 +1,12 @@
+/*
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
+*/
 /*
  * Copyright (c) 1985, 1988, 1990, 1992, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -118,10 +127,10 @@ struct	sockaddr_in his_addr;
 struct	sockaddr_in pasv_addr;
 #endif /* INET6 */
 
-#ifdef IPSEC
+#ifdef NETSEC
 void *request = NULL;
 int requestlen = 0;
-#endif /* IPSEC */
+#endif /* NETSEC */
 
 int	data;
 jmp_buf	errcatch, urgcatch;
@@ -182,7 +191,7 @@ char	proctitle[LINE_MAX];	/* initial part of title */
 		    syslog(LOG_INFO,"%s %s%s", cmd, \
 			*(file) == '/' ? "" : curdir(), file); \
 		else \
-		    syslog(LOG_INFO, "%s %s%s = %qd bytes", \
+		    syslog(LOG_INFO, "%s %s%s = %ld bytes", \
 			cmd, (*(file) == '/') ? "" : curdir(), file, cnt); \
 	}
 
@@ -307,11 +316,11 @@ main(argc, argv, envp)
 	LastArgv = envp[-1] + strlen(envp[-1]);
 #endif /* SETPROCTITLE */
 
-#ifdef IPSEC
+#ifdef NETSEC
 	while ((ch = getopt(argc, argv, "dlt:T:u:vS:")) != EOF) {
-#else /* IPSEC */
+#else /* NETSEC */
 	while ((ch = getopt(argc, argv, "dlt:T:u:v")) != EOF) {
-#endif /* IPSEC */
+#endif /* NETSEC */
 		switch (ch) {
 		case 'd':
 			debug = 1;
@@ -349,14 +358,14 @@ main(argc, argv, envp)
 			debug = 1;
 			break;
 
-#ifdef IPSEC
+#ifdef NETSEC
 		case 'S':
-                        if (nrl_strtoreq(optarg, &request, &requestlen)) {
-                          fprintf(stderr, "ftpd: nrl_strtoreq(%s) failed\n", optarg);
+                        if (netsec_strtoreq(optarg, &request, &requestlen)) {
+                          fprintf(stderr, "ftpd: netsec_strtoreq(%s) failed\n", optarg);
                           exit(1);
                         };
 			break;
-#endif /* IPSEC */
+#endif /* NETSEC */
 
 		default:
 			warnx("unknown flag -%c ignored", optopt);
@@ -382,7 +391,7 @@ main(argc, argv, envp)
 #else /* INET6 */
 	dolog(&his_addr);
 #endif /* INET6 */
-#if IPSEC
+#if NETSEC
         if (!request) {
           requestlen = 2048; /* XXX */
           if (!(request = malloc(requestlen))) {
@@ -394,7 +403,7 @@ main(argc, argv, envp)
 	    exit(1);
           };
         };
-#endif /* IPSEC */
+#endif /* NETSEC */
 	/*
 	 * Set up default state
 	 */
@@ -532,7 +541,11 @@ user(name)
 			guest = 1;
 			askpasswd = 1;
 			reply(331,
+#if INNER
+			    "Guest login ok, send your e-mail address (user@host) as password.");
+#else /* INNER */
 			    "Guest login ok, type your name as password.");
+#endif /* INNER */
 		} else
 			reply(530, "User %s unknown.", name);
 		if (!askpasswd && logging)
@@ -650,6 +663,21 @@ pass(passwd)
 			return;
 		}
 	}
+#if INNER
+        else {
+          char *c = strchr(passwd, '@'), c2;
+          if (!c)
+            goto bogus;
+          if (checkuser(passwd))
+            goto bogus;
+          c++;
+          c2 = *c;
+          *c = 0;
+          if (checkuser(passwd))
+            goto bogus;
+          *c = c2;
+        };
+#endif /* INNER */
 	login_attempts = 0;		/* this time successful */
 	if (setegid((gid_t)pw->pw_gid) < 0) {
 		reply(550, "Can't set gid.");
@@ -728,6 +756,14 @@ pass(passwd)
 bad:
 	/* Forget all about it... */
 	end_login();
+
+#if INNER
+	return;
+
+bogus:
+        reply(550, "No e-mail address, no service.");
+        goto bad;
+#endif /* INNER */
 }
 
 void
@@ -885,12 +921,12 @@ getdatasock(mode)
 #endif /* INET6 */
 	if (s < 0)
 		goto bad;
-#ifdef IPSEC
+#ifdef NETSEC
 	if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
 	  syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
           goto bad;
         };
-#endif /* IPSEC */
+#endif /* NETSEC */
 	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
 	    (char *) &on, sizeof(on)) < 0)
 		goto bad;
@@ -948,7 +984,7 @@ dataconn(name, size, mode)
 	file_size = size;
 	byte_count = 0;
 	if (size != (off_t) -1)
-		(void) sprintf(sizebuf, " (%qd bytes)", size);
+		(void) sprintf(sizebuf, " (%ld bytes)", size);
 	else
 		(void) strcpy(sizebuf, "");
 	if (pdata >= 0) {
@@ -999,8 +1035,8 @@ dataconn(name, size, mode)
 	file = getdatasock(mode);
 	if (file == NULL) {
 #if INET6
-	  char addr[46], port[6];
-	  if (nrl_getnameinfo((struct sockaddr *)&data_source, NRL_SA_LEN((struct sockaddr *)&data_source), addr, sizeof(addr), port, sizeof(port), 3))
+	  char addr[NI_MAXHOST], port[NI_MAXSERV];
+	  if (getnameinfo((struct sockaddr *)&data_source, NRL_SA_LEN((struct sockaddr *)&data_source), addr, sizeof(addr), port, sizeof(port), NI_NUMERICHOST | NI_NUMERICSERV))
 	    reply(425, "Can't create data socket: %s.");
 	  else
 	    reply(425, "Can't create data socket (%s,%s): %s.", addr, port, strerror(errno));
@@ -1242,8 +1278,8 @@ statcmd()
 	printf("     Connected to %s", remotehost);
 #if INET6
         {
-	  char buf[46];
-	  if (!nrl_getnameinfo((struct sockaddr *)&his_addr, NRL_SA_LEN((struct sockaddr *)&his_addr), buf, sizeof(buf), NULL, 0, 1))
+	  char buf[NI_MAXHOST];
+	  if (!getnameinfo((struct sockaddr *)&his_addr, NRL_SA_LEN((struct sockaddr *)&his_addr), buf, sizeof(buf), NULL, 0, NI_NUMERICHOST))
 	    printf(" (%s)", buf);
         }
 #else /* INET6 */
@@ -1511,7 +1547,7 @@ dolog(sin)
 #endif /* INET6 */
 {
 #if INET6
-  if (nrl_getnameinfo(sa, NRL_SA_LEN(sa), remotehost, sizeof(remotehost), NULL, 0, 0)) {
+  if (getnameinfo(sa, NRL_SA_LEN(sa), remotehost, sizeof(remotehost), NULL, 0, NI_NUMERICHOST | NI_NUMERICSERV)) {
     syslog(LOG_ERR, "getnameinfo failed, bailing");
     exit(1);
   }
@@ -1574,10 +1610,10 @@ myoob(signo)
 	}
 	if (strcmp(cp, "STAT\r\n") == 0) {
 		if (file_size != (off_t) -1)
-			reply(213, "Status: %qd of %qd bytes transferred",
+			reply(213, "Status: %ld of %ld bytes transferred",
 			    byte_count, file_size);
 		else
-			reply(213, "Status: %qd bytes transferred", byte_count);
+			reply(213, "Status: %ld bytes transferred", byte_count);
 	}
 }
 
@@ -1607,12 +1643,12 @@ passive()
 		perror_reply(425, "Can't open passive connection");
 		return;
 	}
-#ifdef IPSEC
+#ifdef NETSEC
 	if (setsockopt(pdata, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
 	  syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
           goto pasv_error;
         };
-#endif /* IPSEC */
+#endif /* NETSEC */
 #if INET6
 	memcpy(&pasv_addr, &ctrl_addr, NRL_SA_LEN((struct sockaddr *)&ctrl_addr));
 	switch(ctrl_addr.sa.sa_family) {
@@ -1660,10 +1696,12 @@ passive()
 	      if (satoport(buffer, &pasv_addr))
 		goto pasv_error;
 	      break;
+#if !INNER
 	    case 3:
 	      if (satolport(buffer, &pasv_addr))
 		goto pasv_error;
 	      break;
+#endif /* !INNER */
 	    default:
 	      goto pasv_error;
 	  }
@@ -1754,11 +1792,11 @@ send_file_list(whichf)
 	glob_t gl;
 
 	if (strpbrk(whichf, "~{[*?") != NULL) {
-#ifdef GLOB_BRACE
+#if defined(GLOB_BRACE) && defined(GLOB_QUOTE) && defined(GLOB_TILDE)
 		int flags = GLOB_BRACE|GLOB_NOCHECK|GLOB_QUOTE|GLOB_TILDE;
-#else /* GLOB_BRACE */
+#else /* defined(GLOB_BRACE) && defined(GLOB_QUOTE) && defined(GLOB_TILDE) */
 		int flags = GLOB_NOCHECK;
-#endif /* GLOB_BRACE */
+#endif /* defined(GLOB_BRACE) && defined(GLOB_QUOTE) && defined(GLOB_TILDE) */
 
 		memset(&gl, 0, sizeof(gl));
 		freeglob = 1;
