@@ -1,6 +1,6 @@
 /*
-%%% copyright-cmetz
-This software is Copyright 1996 by Craig Metz, All Rights Reserved.
+%%% copyright-cmetz-96
+This software is Copyright 1996-1997 by Craig Metz, All Rights Reserved.
 The Inner Net License Version 2 applies to this software.
 You should have received a copy of the license with this software. If
 you didn't get a copy, you may request one from <license@inner.net>.
@@ -9,6 +9,7 @@ you didn't get a copy, you may request one from <license@inner.net>.
 #ifndef _NETINET6_IN6_H
 #define _NETINET6_IN6_H 1
 
+#include <sys/types.h>
 #include <linux/in6.h>
 #include <linux/ipv6.h>
 
@@ -18,35 +19,20 @@ extern const struct in6_addr in6addr_any;
 extern const struct in6_addr in6addr_loopback;
 #define IN6ADDR_LOOPBACK_INIT { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 }
 
-#ifdef notyet
-unsigned int if_nametoindex(const char *ifname);
-char *if_indextoname(unsigned int ifindex, char *ifname);
-
-struct if_nameindex {
-  unsigned int  if_index;
-  char         *if_name;   /* null terminated name: "le0", ... */
-};
-
-struct if_nameindex *if_nameindex(void);
-#endif /* notyet */
-
 #define INET_ADDRSTRLEN 16
 #define INET6_ADDRSTRLEN 46
 
-int inet6_isipv4mapped(const struct in6_addr *addr);
+int inet_pton(int, const char *, void *);
+const char *inet_ntop(int, const void *, char *, size_t);
 
-int inet_pton __P((int, const char *, void *));
-const char *inet_ntop __P((int, const void *, char *, size_t));
-int getnameinfo __P((const struct sockaddr *sa, size_t addrlen, char *host, size_t hostlen, char *serv, size_t servlen));
-
-#ifndef IPPROTO_NONE
+#ifndef IPPROTO_HOP
 #define IPPROTO_HOP       0
 #define IPPROTO_IPV4      4
 #define IPPROTO_ROUTING  43
 #define IPPROTO_FRAGMENT 44
 #define IPPROTO_NONE     59
 #define IPPROTO_DST      60
-#endif /* IPPROTO_NONE */
+#endif /* IPPROTO_HOP */
 
 #define IN6_IS_ADDR_UNSPECIFIED(a) \
         ((((u_int32_t *)(a))[0] == 0) && ((u_int32_t *)(a))[1] == 0) && \
