@@ -1,3 +1,12 @@
+/*
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
+*/
 /*
  * inetd.c  --  Internet super-server.  Now with IPv6 support.
  *
@@ -234,10 +243,10 @@ struct	servtab {
 	short	se_wait;		/* single threaded server */
 	short	se_checked;		/* looked at during merge */
 	char	*se_user;		/* user name to run as */
-#ifdef IPSEC
+#ifdef NETSEC
         void    *se_request;
         int     se_requestlen;
-#endif /* IPSEC */
+#endif /* NETSEC */
 	struct	biltin *se_bi;		/* if built-in, description */
 	char	*se_server;		/* server program */
 #define	MAXARGV 20
@@ -636,7 +645,7 @@ config(signo)
 			for (i = 0; i < MAXARGV; i++)
 				SWAP(sep->se_argv[i], cp->se_argv[i]);
 
-#ifdef IPSEC
+#ifdef NETSEC
 			if ((cp->se_requestlen != sep->se_requestlen) || (cp->se_request && memcmp(cp->se_request, sep->se_request, cp->se_requestlen))) {
 			  if (sep->se_request)
 			    free(sep->se_request);
@@ -651,7 +660,7 @@ config(signo)
 			      exit(1);
 			    };
 			};
-#endif /* IPSEC */
+#endif /* NETSEC */
 			sigsetmask(omask);
 			freeconfig(cp);
 			if (debug)
@@ -761,7 +770,7 @@ setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
 	};
 #endif /* OFFSETPORT */
 
-#ifdef IPSEC
+#ifdef NETSEC
 	if (sep->se_request && (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0)) {
 	  if (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0) {
 	    syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", strerror(errno), errno);
@@ -770,7 +779,7 @@ setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
 	    exit(1);
 	  };
 	};
-#endif /* IPSEC */
+#endif /* NETSEC */
 
 	if (bind(sep->se_fd, (struct sockaddr *)&sep->se_ctrladdr,
 #if INET6
@@ -955,9 +964,9 @@ more:
 	  sprintf(buffer, "%s.%s", host ? host : "*", serv);
 
 	  if ((i = getaddrinfo(host, serv, &req, &ai))) {
-	    syslog(LOG_ERR, "getaddrinfo %s: %s", sep->se_service, nrl_gai_strerror(i));
+	    syslog(LOG_ERR, "getaddrinfo %s: %s(%d)", sep->se_service, gai_strerror(i), i);
             if (debug)
-	      fprintf(stderr, "getaddrinfo %s: %s\n", sep->se_service, nrl_gai_strerror(i));
+	      fprintf(stderr, "getaddrinfo %s: %s(%d)\n", sep->se_service, gai_strerror(i), i);
 	    exit(1);
 	  };
 
@@ -994,7 +1003,7 @@ more:
 	}
 	sep->se_user = newstr(sskip(&cp));
 
-#ifdef IPSEC
+#ifdef NETSEC
 	{
 	  char *c = newstr(sskip(&cp));
 
@@ -1002,15 +1011,15 @@ more:
 	  sep->se_requestlen = 0;
 
 	  if (strcmp(c, "*")) {
-	    if (nrl_strtoreq(c, &sep->se_request, &sep->se_requestlen)) {
-	      syslog(LOG_ERR, "nrl_strtoreq(%s) failed", c);
+	    if (netsec_strtoreq(c, &sep->se_request, &sep->se_requestlen)) {
+	      syslog(LOG_ERR, "netsec_strtoreq(%s) failed", c);
 	      exit(1);
 	    };
 	  };
 
 	  free(c);
 	};
-#endif /* IPSEC */
+#endif /* NETSEC */
 
 	sep->se_server = newstr(sskip(&cp));
 	if (strcmp(sep->se_server, "internal") == 0) {
@@ -1170,7 +1179,7 @@ inetd_setproctitle(a, s)
 
 	  i = strlen(buf);
 
-	  if (nrl_getnameinfo((struct sockaddr *)&sa, size, buf + i, sizeof(buf) - i, NULL, 0, nflag)) {
+	  if (getnameinfo((struct sockaddr *)&sa, size, buf + i, sizeof(buf) - i, NULL, 0, nflag ? NI_NUMERICHOST : 0)) {
 	    syslog(LOG_ERR, "getnameinfo failed");
 	    buf[i - 1] = 0;
 	  } else
@@ -1441,7 +1450,7 @@ print_service(action, sep)
 	char *action;
 	struct servtab *sep;
 {
-#if IPSEC
+#if NETSEC
         char buffer[64];
 
 	if (sep->se_request) {
@@ -1458,13 +1467,13 @@ print_service(action, sep)
 	    sep->se_wait, sep->se_user, (long)sep->se_bi,
 	    buffer,
 	    sep->se_server);
-#else /* IPSEC */
+#else /* NETSEC */
 	fprintf(stderr,
 	    "%s: %s proto=%s, wait=%d, user=%s builtin=%x server=%s\n",
 	    action, sep->se_service, sep->se_proto,
 	    sep->se_wait, sep->se_user, (int)sep->se_bi, 
 	    sep->se_server);
-#endif /* IPSEC */
+#endif /* NETSEC */
 }
 
 /*
