@@ -1,3 +1,12 @@
+/*
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
+*/
 /*
  * socktest.c  --  Socket testing program.  Useful for first-cut tests on
  *                 AF_{INET,INET6}.
@@ -96,12 +105,6 @@ Research Laboratory (NRL).
 extern char *optarg;
 extern int errno, optind;
 
-struct nrl_nametonum socktypes[] = {
-  { SOCK_STREAM, "stream", 0 },
-  { SOCK_DGRAM, "dgram", 0 },
-  { 0, NULL, 0 }
-};
-
 char *portname = NULL;
 char *hostname = NULL;
 
@@ -123,11 +126,11 @@ typedef union {
 char addrbuf[sizeof(su)], addrbuf2[sizeof(su)];
 int addrlen;
 
-#if INET6 && defined(IPV6_RXINFO)
+#if INET6 && defined(IPV6_PKTINFO)
 struct in6_pktinfo in6_pktinfo;
-#endif /* INET6 && defined(IPV6_RXINFO) */
+#endif /* INET6 && defined(IPV6_PKTINFO) */
 
-char controlbuf[1024];
+char controlbuf[CMSG_SPACE(sizeof(struct in6_pktinfo))];
 
 char *datablock;
 int datablocklen = 65535;
@@ -207,9 +210,9 @@ usage(char *myname)
 #ifdef ADDRFORM
   fprintf(stderr, " [-f address_form]");
 #endif /* ADDRFORM */
-#if IPSEC
+#if NETSEC
   fprintf(stderr, " [-S security_request]");
-#endif /* IPSEC */
+#endif /* NETSEC */
   fprintf(stderr," [-s] [host] [port]\n");
   exit(1);
 }
@@ -248,16 +251,19 @@ main(argc, argv)
       }
       break;
 #endif /* ADDRFORM */
-#if IPSEC
+#if NETSEC
     case 'S':
-      requeststr = optarg;
+      if (netsec_strtoreq(optarg, &request, &requestlen)) {
+	fprintf(stderr, "netsec_strtoreq(%s, ...) failed!\n", optarg);
+	exit(1);
+      };
       break;
-#endif /* IPSEC */
+#endif /* NETSEC */
     case 's':
       server = 1;
       break;
     case 't':
-      if ((type = nrl_nametonum(socktypes, optarg)) == -1) {
+      if ((type = nrl_socktypenametonum(optarg)) == -1) {
 	fprintf(stderr, "%s: invalid socket type: %s\n", myname, optarg);
 	exit(1);
       }
@@ -293,7 +299,7 @@ main(argc, argv)
   if (!portname)
     portname = "7777";
 
-#if INET6 && defined(IPV6_TXINFO)
+#if INET6 && defined(IPV6_PKTINFO)
   if (optind < argc)
     if (!(in6_pktinfo.ipi6_ifindex = if_nametoindex(argv[optind]))) {
       fprintf(stderr, "%s: %s: unknown/invalid interface name\n", myname, argv[optind]);
@@ -306,12 +312,12 @@ main(argc, argv)
     memset(&req, 0, sizeof(struct addrinfo));
     req.ai_family = AF_INET6;
     if (i = getaddrinfo(argv[optind], NULL, &req, &ai)) {
-      fprintf(stderr, "%s: getaddrinfo: %s.%s: %s\n", myname, argv[optind], NULL, nrl_gai_strerror(i));
+      fprintf(stderr, "%s: getaddrinfo: %s.%s: %s(%d)\n", myname, argv[optind], NULL, gai_strerror(i), i);
       exit(1);
     };
     in6_pktinfo.ipi6_addr = ((struct sockaddr_in6 *)(ai->ai_addr))->sin6_addr;
   };
-#endif /* INET6 && defined(IPV6_TXINFO) */
+#endif /* INET6 && defined(IPV6_PKTINFO) */
 
   memset(&req, 0, sizeof(struct addrinfo));
 
@@ -321,7 +327,7 @@ main(argc, argv)
     req.ai_flags |= AI_PASSIVE;
 
   if (i = getaddrinfo(hostname, portname, &req, &ai)) {
-    fprintf(stderr, "%s: getaddrinfo: %s.%s: %s\n", myname, hostname, portname, nrl_gai_strerror(i));
+    fprintf(stderr, "%s: getaddrinfo: %s.%s: %s(%d)\n", myname, hostname, portname, gai_strerror(i), i);
     exit(1);
   }
 
@@ -330,7 +336,7 @@ main(argc, argv)
 #endif /* FASTCTO */
 
   for (ai2 = ai, s = -1; ai; ai = ai->ai_next) {
-    if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), 0)) {
+    if (getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), 0)) {
       printf("%s: getnameinfo() failed!\n", myname);
       continue;
     }
@@ -342,16 +348,11 @@ main(argc, argv)
       continue;
     }
 
-#if IPSEC
+#if NETSEC
     if (requeststr) {
       void *request, *request2;
       int requestlen, requestlen2;
 
-      if (nrl_strtoreq(requeststr, &request, &requestlen)) {
-	fprintf(stderr, "nrl_strtoreq() failed!\n");
-	exit(1);
-      };
-      
       if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
 	fprintf(stderr, "%s: setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", myname, strerror(errno), errno);
 	exit(1);
@@ -386,7 +387,7 @@ main(argc, argv)
       free(request2);
       free(request);
     };
-#endif /* IPSEC */
+#endif /* NETSEC */
 
     if (ai->ai_flags & AI_PASSIVE) {
       if (bind(s, ai->ai_addr, ai->ai_addrlen) < 0) {
@@ -445,17 +446,17 @@ main(argc, argv)
     addrform(aform);
 #endif /* INET6 && defined(IPV6_ADDRFORM) */
 
-#if INET6 && defined(IPV6_RXINFO)
+#if INET6 && defined(IPV6_PKTINFO)
   if (af == AF_INET6) {
     int val = 1;
-    if (setsockopt(s, IPPROTO_IPV6, IPV6_RXINFO, &val, sizeof(val)) < 0) {
-      perror("setsockopt (IPV6_RXINFO)");
+    if (setsockopt(s, IPPROTO_IPV6, IPV6_PKTINFO, &val, sizeof(val)) < 0) {
+      perror("setsockopt (IPV6_PKTINFO)");
       exit(1);
     }
   };
-#endif /* INET6 && defined(IPV6_RXINFO) */
+#endif /* INET6 && defined(IPV6_PKTINFO) */
 
-  if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag)) {
+  if (getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag ? (NI_NUMERICHOST | NI_NUMERICSERV) : 0)) {
     fprintf(stderr, "getnameinfo() failed!\n");
     exit(1);
   }
@@ -487,15 +488,25 @@ main(argc, argv)
   if (server)
     goto servercode;
 
+  {
+    i = 1;
+    if ((setsockopt(s, SOL_SOCKET, SO_BROADCAST, &i, sizeof(int)) < 0) && (errno != EPERM)) {
+      fprintf(stderr, "%s: setsockopt(..., SO_BROADCAST, ...): %s(%d)\n", myname, strerror(errno), errno);
+      exit(1);
+    };
+  };
+
   if (ai->ai_family == AF_INET6) {
     msghdr.msg_control = controlbuf;
-    msghdr.msg_controllen = sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo);
+    msghdr.msg_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
 
-    ((struct cmsghdr *)controlbuf)->cmsg_len = sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo);
+    memset(controlbuf, 0, sizeof(controlbuf));
+
+    ((struct cmsghdr *)controlbuf)->cmsg_len = CMSG_LEN(sizeof(struct in6_pktinfo));
     ((struct cmsghdr *)controlbuf)->cmsg_level = IPPROTO_IPV6;
-    ((struct cmsghdr *)controlbuf)->cmsg_type = IPV6_RXINFO;
+    ((struct cmsghdr *)controlbuf)->cmsg_type = IPV6_PKTINFO;
 
-    memcpy(controlbuf + sizeof(struct cmsghdr), &in6_pktinfo, sizeof(struct in6_pktinfo));
+    memcpy(CMSG_DATA((struct cmsghdr *)controlbuf), &in6_pktinfo, sizeof(struct in6_pktinfo));
   } else {
     msghdr.msg_control = NULL;
     msghdr.msg_controllen = 0;
@@ -590,7 +601,7 @@ servercode:
 
       addrlen = sizeof(addrbuf);
       if ((s2 = accept(s, (struct sockaddr *)addrbuf, &addrlen)) >= 0) {
-	if (nrl_getnameinfo((struct sockaddr *)addrbuf, addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag)) {
+	if (getnameinfo((struct sockaddr *)addrbuf, addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag ? (NI_NUMERICHOST | NI_NUMERICSERV) : 0)) {
 	  fprintf(stderr, "getnameinfo() failed!\n");
 	  exit(1);
 	}
@@ -619,17 +630,17 @@ servercode:
     if (datasize && memcmp(addrbuf, addrbuf2, sizeof(addrbuf))) {
       memcpy(addrbuf2, addrbuf, sizeof(addrbuf));
 
-      if (nrl_getnameinfo((struct sockaddr *)addrbuf, msghdr2.msg_namelen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag)) {
+      if (getnameinfo((struct sockaddr *)addrbuf, msghdr2.msg_namelen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag ? (NI_NUMERICHOST | NI_NUMERICSERV) : 0)) {
 	fprintf(stderr, "getnameinfo() failed!\n");
 	exit(1);
       };
     };
 
-#if INET6 && defined(IPV6_RXINFO)
+#if INET6 && defined(IPV6_PKTINFO)
     {
       struct cmsghdr *cmsghdr;
       for (cmsghdr = CMSG_FIRSTHDR(&msghdr2); cmsghdr; cmsghdr = CMSG_NXTHDR(&msghdr2, cmsghdr))
-	if ((cmsghdr->cmsg_level == IPPROTO_IPV6) && (cmsghdr->cmsg_type == IPV6_RXINFO))
+	if ((cmsghdr->cmsg_level == IPPROTO_IPV6) && (cmsghdr->cmsg_type == IPV6_PKTINFO))
 	  if (memcmp(CMSG_DATA(cmsghdr), &in6_pktinfo, sizeof(struct in6_pktinfo))) {
 	    struct sockaddr_in6 sin6;
 	    memcpy(&in6_pktinfo, CMSG_DATA(cmsghdr), sizeof(struct in6_pktinfo));
@@ -643,13 +654,13 @@ servercode:
 #endif /* SALEN */
             sin6.sin6_family = AF_INET6;
 	    sin6.sin6_addr = in6_pktinfo.ipi6_addr;
-	    if (nrl_getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), dbuf, sizeof(dbuf), NULL, 0, nflag)) {
+	    if (getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), dbuf, sizeof(dbuf), NULL, 0, nflag ? NI_NUMERICHOST : 0)) {
 	      fprintf(stderr, "getnameinfo() failed!\n");
 	      exit(1);
 	    };
 	  };
     };
-#endif /* INET6 && defined(IPV6_RXINFO) */
+#endif /* INET6 && defined(IPV6_PKTINFO) */
 
     if (datasize) {
       datablock[datasize] = 0;
