@@ -1,3 +1,12 @@
+/*
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
+*/
 /*
  * ping.c  --  Implement the ping(8) command.
  *
@@ -149,7 +158,6 @@ static char sccsid[] = "@(#)ping.c	8.3 (Berkeley) 4/28/95";
 #include <netinet/in.h>
 #include <netinet/ip.h>
 #include <netinet/ip_icmp.h>
-#include <netinet/ip_var.h>
 #include <netdb.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -167,9 +175,9 @@ static char sccsid[] = "@(#)ping.c	8.3 (Berkeley) 4/28/95";
 #include "support.h"
 #endif /* INET6 */
 
-#if IPSEC
+#if NETSEC
 #include <netsec/ipsec.h>
-#endif /* IPSEC */
+#endif /* NETSEC */
 
 #define	DEFDATALEN	(64 - 8)	/* default data length */
 #define	MAXIPLEN	60
@@ -223,8 +231,14 @@ char BSPACE = '\b';		/* characters written for flood */
 char DOT = '.';
 char *hostname;
 int ident;			/* process id to identify our packets */
+#if INET6
 int af = 0;                     /* address family */
 
+struct in6_pktinfo send_pktinfo;
+char send_controlbuf[CMSG_SPACE(sizeof(struct in6_pktinfo))];
+int send_controllen = 0;
+#endif /* INET6 */
+
 /* counters */
 long npackets;			/* max packets to transmit */
 long nreceived;			/* # of packets we got back */
@@ -261,10 +275,10 @@ main(argc, argv)
 	int ch, fdmask, hold, packlen, preload;
 	u_char *datap, *packet;
 	char *target, hnamebuf[MAXHOSTNAMELEN], *malloc();
-#if IPSEC
+#if NETSEC
 	void *request = NULL;
 	int requestlen = 0;
-#endif /* IPSEC */
+#endif /* NETSEC */
 #ifdef IP_OPTIONS
 	char rspace[3 + 4 * NROUTES + 1];	/* record route space */
 #endif
@@ -272,31 +286,46 @@ main(argc, argv)
 
 	preload = 0;
 	datap = &outpack[8 + sizeof(struct timeval)];
+#if INET6
+	memset(&send_pktinfo, 0, sizeof(struct in6_pktinfo));
+#endif /* INET6 */
 	while ((ch = getopt(argc, argv,
 #if INET6
-			    "a:"
+			    "a:I:"
 #endif /* INET6 */
-#if IPSEC
+#if NETSEC
 			    "S:"
-#endif /* IPSEC */
+#endif /* NETSEC */
 			    "Rc:dfa:hi:l:np:qrs:v")) != EOF)
 		switch(ch) {
 #if INET6
 		case 'a':
 		  if ((af = nrl_afnametonum(optarg)) < 0) {
-		    (void)fprintf(stderr, "ping: %s: invalid family\n", optarg);
+		    fprintf(stderr, "ping: %s: invalid family\n", optarg);
 		    exit(1);
 		  }
 		  break;
+		case 'I':
+		  if (!(send_pktinfo.ipi6_ifindex = if_nametoindex(optarg))) {
+		    fprintf(stderr, "ping: %s: invalid interface\n", optarg);
+		    exit(1);
+		  };
+                  memset(send_controlbuf, 0, sizeof(send_controlbuf));
+		  ((struct cmsghdr *)send_controlbuf)->cmsg_len = CMSG_LEN(sizeof(struct in6_pktinfo));
+		  ((struct cmsghdr *)send_controlbuf)->cmsg_level = IPPROTO_IPV6;
+		  ((struct cmsghdr *)send_controlbuf)->cmsg_type = IPV6_PKTINFO;
+		  memcpy(CMSG_DATA((struct cmsghdr *)send_controlbuf), &send_pktinfo, sizeof(struct in6_pktinfo));
+		  send_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
+		  break;
 #endif /* INET6 */
-#if IPSEC
+#if NETSEC
 		case 'S':
-                  if (nrl_strtoreq(optarg, &request, &requestlen)) {
-                    fprintf(stderr, "ping: nrl_strtoreq(%s) failed\n", optarg);
+                  if (netsec_strtoreq(optarg, &request, &requestlen)) {
+                    fprintf(stderr, "ping: netsec_strtoreq(%s, ...) failed\n", optarg);
                     exit(1);
                   };
                   break;
-#endif /* IPSEC */
+#endif /* NETSEC */
 		case 'c':
 			npackets = atoi(optarg);
 			if (npackets <= 0) {
@@ -385,7 +414,7 @@ main(argc, argv)
 	  req.ai_family = af;
 
 	  if (i = getaddrinfo(hostname = target, NULL, &req, &ai)) {
-	    fprintf(stderr, "ping: %s: %s\n", target, nrl_gai_strerror(i));
+	    fprintf(stderr, "ping: %s: %s\n", target, gai_strerror(i));
 	    exit(1);
 	  };
 
@@ -466,13 +495,13 @@ main(argc, argv)
 		perror("ping: socket");
 		exit(1);
 	}
-#if IPSEC
+#if NETSEC
 	if (request)
           if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
             fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
 	    exit(1);
 	  };
-#endif /* IPSEC */
+#endif /* NETSEC */
 
 	hold = 1;
 	if (options & F_SO_DEBUG)
@@ -481,6 +510,7 @@ main(argc, argv)
 	if (options & F_SO_DONTROUTE)
 		(void)setsockopt(s, SOL_SOCKET, SO_DONTROUTE, (char *)&hold,
 		    sizeof(hold));
+	setsockopt(s, SOL_SOCKET, SO_BROADCAST, &hold, sizeof(hold));
 
 #if INET6
 	if (af == AF_INET) {
@@ -550,7 +580,7 @@ main(argc, argv)
         {
 	  char buf[64];
 
-	  if (nrl_getnameinfo((struct sockaddr *)&whereto, NRL_SA_LEN((struct sockaddr *)&whereto), buf, sizeof(buf), NULL, 0, 1)) {
+	  if (getnameinfo((struct sockaddr *)&whereto, NRL_SA_LEN((struct sockaddr *)&whereto), buf, sizeof(buf), NULL, 0, NI_NUMERICHOST)) {
 	    fprintf(stderr, "ping: getnameinfo failed\n");
 	    exit(1);
 	  }
@@ -691,8 +721,12 @@ pinger()
 	register struct icmp *icp;
 #if INET6
 	register struct icmpv6hdr *icmpv6 = (struct icmpv6hdr *)outpack;
-#endif /* INET6 */
+	struct msghdr msghdr;
+	struct iovec iovec;
+
+#else /* INET6 */
 	register int cc;
+#endif /* INET6 */
 	int i;
 
 	icp = (struct icmp *)outpack;
@@ -703,6 +737,23 @@ pinger()
 	 * is identical to IPv4, but this is cleaner at the cost of  
 	 * additional code
 	 */
+
+	memset(&msghdr, 0, sizeof(struct msghdr));
+	msghdr.msg_name = (caddr_t)&whereto;
+	msghdr.msg_namelen = NRL_SA_LEN((struct sockaddr *)&whereto);
+	msghdr.msg_iov = &iovec;
+	msghdr.msg_iovlen = 1;
+	if (send_controllen) {
+	  msghdr.msg_control = send_controlbuf;
+	  msghdr.msg_controllen = send_controllen;
+	};
+
+	memset(&iovec, 0, sizeof(struct iovec));
+	iovec.iov_base = (caddr_t)outpack;
+
+	if (timing)
+	  gettimeofday((struct timeval *)&outpack[8], (struct timezone *)NULL);
+
 	switch(af) {
 	  case AF_INET:
 #endif /* INET6 */
@@ -713,6 +764,8 @@ pinger()
 	icp->icmp_id = ident;			/* ID */
 	CLR(icp->icmp_seq % mx_dup_ck);
 #if INET6
+	    iovec.iov_len = datalen + ICMP_MINLEN;
+	    icp->icmp_cksum = in_cksum((u_short *)icp, iovec.iov_len);
 	    break;
 	  case AF_INET6:
 	    icmpv6->icmpv6_type = ICMPV6_ECHO_REQUEST;
@@ -721,38 +774,30 @@ pinger()
 	    icmpv6->icmpv6_id = ident;
 	    icmpv6->icmpv6_seq = ntransmitted++;
 	    CLR(icmpv6->icmpv6_seq % mx_dup_ck);
+	    iovec.iov_len = datalen + sizeof(struct icmpv6hdr);
 	    break;
 	  default:
 	    fprintf(stderr, "ping: Don't know how to build an echo request for %s\n", nrl_afnumtoname(af));
 	    exit(1);
 	} 
-#endif /* INET6 */
 
+	if ((i = sendmsg(s, &msghdr, 0)) != iovec.iov_len) {
+	  if (i < 0)
+	    perror("ping: sendmsg");
+	  printf("ping: wrote %s %d chars, ret=%d\n", hostname, iovec.iov_len, i);
+	};
+#else /* INET6 */
 	if (timing)
 		(void)gettimeofday((struct timeval *)&outpack[8],
 		    (struct timezone *)NULL);
 
 	cc = datalen + 8;			/* skips ICMP portion */
 
-#if INET6
-	/*
-	 * compute IPv4 ICMP checksum here.
-	 * for IPv6, just let the kernel do the checksum calculation in 
-	 * ipv6_output, since the checksum now includes an ipv6 pseudo 
-	 * header, which requires us to know the source address of the
-	 * sending interface. 
-	 */  
-	if (af == AF_INET)
-	  icp->icmp_cksum = in_cksum((u_short *)icp, cc);
-
-	i = sendto(s, (char *)outpack, cc, 0, (struct sockaddr *)&whereto, NRL_SA_LEN((struct sockaddr *)&whereto));
-#else /* INET6 */
 	/* compute ICMP checksum here */
 	icp->icmp_cksum = in_cksum((u_short *)icp, cc);
 
 	i = sendto(s, (char *)outpack, cc, 0, (struct sockaddr *)&whereto,
 	    sizeof(struct sockaddr));
-#endif /* INET6 */
 
 	if (i < 0 || i != cc)  {
 		if (i < 0)
@@ -760,6 +805,7 @@ pinger()
 		(void)printf("ping: wrote %s %d chars, ret=%d\n",
 		    hostname, cc, i);
 	}
+#endif /* INET6 */
 	if (!(options & F_QUIET) && options & F_FLOOD)
 		(void)write(STDOUT_FILENO, &DOT, 1);
 }
@@ -788,6 +834,9 @@ pr_pack(buf, cc, from)
 	register int i, j;
 	register u_char *cp,*dp;
 	static int old_rrlen;
+#ifndef MAX_IPOPTLEN
+#define MAX_IPOPTLEN 44
+#endif /* MAX_IPOPTLEN */
 	static char old_rr[MAX_IPOPTLEN];
 #if !INET6
 	struct ip *ip;
@@ -824,7 +873,7 @@ pr_pack(buf, cc, from)
 	    break;
 	};
 
-        if (nrl_getnameinfo(from, NRL_SA_LEN(from), name, sizeof(name), NULL, 0, 1)) {
+        if (getnameinfo(from, NRL_SA_LEN(from), name, sizeof(name), NULL, 0, NI_NUMERICHOST)) {
           fprintf(stderr, "ping: getnameinfo failed\n");
 	  exit(1);
 	};
@@ -1409,9 +1458,9 @@ pr_iph(ip)
 	    ip->ip_v, ip->ip_hl, ip->ip_tos, ip->ip_len, ip->ip_id);
 	(void)printf("   %1x %04x", ((ip->ip_off) & 0xe000) >> 13,
 	    (ip->ip_off) & 0x1fff);
-#ifdef linux
+#if defined(linux) && !defined(__GLIBC__)
 #define ip_sum ip_csum
-#endif /* linux */
+#endif /* defined(linux) && !defined(__GLIBC__) */
 	(void)printf("  %02x  %02x %04x", ip->ip_ttl, ip->ip_p, ip->ip_sum);
 	(void)printf(" %s ", inet_ntoa(*(struct in_addr *)&ip->ip_src.s_addr));
 	(void)printf(" %s ", inet_ntoa(*(struct in_addr *)&ip->ip_dst.s_addr));
@@ -1435,13 +1484,13 @@ pr_iph(ip)
           sin6.sin6_family = AF_INET6;
 
           sin6.sin6_addr = ipv6->ipv6_src;
-          if (nrl_getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), hbuf1, sizeof(hbuf1), NULL, 0, (options & F_NUMERIC) ? 1 : 0)) {
+          if (getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), hbuf1, sizeof(hbuf1), NULL, 0, (options & F_NUMERIC) ? NI_NUMERICHOST : 0)) {
             fprintf(stderr, "ping: getnameinfo failed\n");
             exit(1);
           };
 
           sin6.sin6_addr = ipv6->ipv6_dst;
-          if (nrl_getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), hbuf2, sizeof(hbuf2), NULL, 0, (options & F_NUMERIC) ? 1 : 0)) {
+          if (getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), hbuf2, sizeof(hbuf2), NULL, 0, (options & F_NUMERIC) ? NI_NUMERICHOST : 0)) {
             fprintf(stderr, "ping: getnameinfo failed\n");
             exit(1);
           };
@@ -1599,9 +1648,9 @@ usage()
 #if INET6
 	    "[-a family] "
 #endif /* INET6 */
-#if IPSEC
+#if NETSEC
 	    "[-S security request] "
-#endif /* IPSEC */
+#endif /* NETSEC */
 	    "[-c count] [-i wait] [-l preload] [-p pattern] [-s packetsize] host\n");
 	exit(1);
 }
