@@ -1,8 +1,12 @@
-/* This software is Copyright 1996 by Craig Metz, All Rights Reserved.
-   The Inner Net Copyright Notice and License Agreement Version 2
-   applies to this software.
+/*
+%%% copyright-cmetz-96
+This software is Copyright 1996-1997 by Craig Metz, All Rights Reserved.
+The Inner Net License Version 2 applies to this software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
 */
-/* gai v1.11 */
+/* gai v1.20 */
 #define HAVE_NRL 1
 
 #include <stdio.h>
@@ -41,7 +45,7 @@ static struct nrl_nametonum socktypenametonum[] = {
 int main(int argc, char **argv)
 {
   int i, j;
-  struct addrinfo *pai = NULL, **ppai = &pai;
+  struct addrinfo *ai = NULL, **pai = &ai;
   struct addrinfo req;
   char *name = NULL, *service = NULL;
   char *myname = strdup(argv[0]);
@@ -102,7 +106,7 @@ int main(int argc, char **argv)
       req.ai_flags |= AI_PASSIVE;
       break;
     case 'x':
-      ppai = NULL;
+      pai = NULL;
       break;
     default:
       usage(argv[0]);
@@ -110,35 +114,34 @@ int main(int argc, char **argv)
 /* gai v1.00 */
   }
 
-  i = getaddrinfo(name, service, &req, ppai);
+  i = getaddrinfo(name, service, &req, pai);
   
-  printf("getaddrinfo() returned: %s(%d).\n", nrl_gai_strerror(i), i);
+  printf("getaddrinfo() returned: %s(%d).\n", gai_strerror(i), i);
   
   if (!i) {
     i = 0;
-    while(pai) {
+    for (; ai; ai = ai->ai_next) {
       printf("\nrecord %d:\n", i++);
-      printf("ai_flags:      %x(%d)\n", pai->ai_flags, pai->ai_flags);
-      printf("ai_family:     %x(%d)\n", pai->ai_family, pai->ai_family);
-      printf("ai_socktype:   %x(%d)\n", pai->ai_socktype, pai->ai_socktype);
-      printf("ai_protocol:   %x(%d)\n", pai->ai_protocol, pai->ai_protocol);
-      printf("ai_addrlen:    %x(%d)\n", pai->ai_addrlen, pai->ai_addrlen);
-      if (getnameinfo(pai->ai_addr, pai->ai_addrlen, hbuf[0], sizeof(hbuf[0]), sbuf[0], sizeof(sbuf[0]))) {
+      printf("ai_flags:      %x(%d)\n", ai->ai_flags, ai->ai_flags);
+#if HAVE_NRL
+      printf("ai_family:     %s(%d)\n", nrl_afnumtoname(ai->ai_family), ai->ai_family);
+      printf("ai_socktype:   %s(%d)\n", nrl_socktypenumtoname(ai->ai_socktype), ai->ai_socktype);
+#else /* HAVE_NRL */
+      printf("ai_family:     %x(%d)\n", ai->ai_family, ai->ai_family);
+      printf("ai_socktype:   %x(%d)\n", ai->ai_socktype, ai->ai_socktype);
+#endif /* HAVE_NRL */
+      printf("ai_protocol:   %x(%d)\n", ai->ai_protocol, ai->ai_protocol);
+      printf("ai_addrlen:    %x(%d)\n", ai->ai_addrlen, ai->ai_addrlen);
+      if (getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf[0], sizeof(hbuf[0]), sbuf[0], sizeof(sbuf[0]), 0)) {
         fprintf(stderr, "getnameinfo() failed!\n");
 	continue;
       };
-#if HAVE_NRL
-      if (nrl_getnameinfo(pai->ai_addr, pai->ai_addrlen, hbuf[1], sizeof(hbuf[1]), sbuf[1], sizeof(sbuf[1]), 7)) {
-        fprintf(stderr, "nrl_getnameinfo() failed!\n");
+      if (getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf[1], sizeof(hbuf[1]), sbuf[1], sizeof(sbuf[1]), NI_NUMERICHOST | NI_NUMERICSERV)) {
+        fprintf(stderr, "getnameinfo() failed!\n");
 	continue;
       };
       printf("ai_addr:       %s.%s(%s.%s)\n", hbuf[0], sbuf[0], hbuf[1], sbuf[1]);
-#else /* HAVE_NRL */
-      printf("ai_addr:       %s.%s\n", hbuf[0], sbuf[0]);
-#endif /* HAVE_NRL */
-      printf("ai_canonname:  %s\n", pai->ai_canonname);
-
-      pai = pai->ai_next;
+      printf("ai_canonname:  %s\n", ai->ai_canonname);
     }
   }
 
