@@ -1,3 +1,12 @@
+/*
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+
+*/
 /*
  * Copyright 1995 by Randall Atkinson, Bao Phan, and Dan McDonald
  *	All Rights Reserved.  
@@ -76,7 +85,7 @@ Research Laboratory (NRL).
 
 #define min(a,b) ((a) <= (b) ? (a) : (b))
 
-static char buffer[6];
+static char buffer[12];
 static char hextab[] = { '0', '1', '2', '3', '4', '5', '6', '7',
                          '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
 
@@ -90,31 +99,38 @@ static struct nrl_nametonum afnametonum[] = {
   { 0, NULL, -1 }
 };
 
-static struct nrl_nametonum gainametonum[] = {
-  { 0, "success", 0 },
-  { EAI_BADFLAGS, "invalid value for ai_flags", 0 },
-  { EAI_NONAME, "name or service is not known", 0 },
-  { EAI_AGAIN, "temporary failure in name resolution", 0 },
-  { EAI_FAIL, "non-recoverable failure in name resolution", 0 },
-  { EAI_NODATA, "no address associated with name", 0 },
-  { EAI_FAMILY, "ai_family not supported", 0 },
-  { EAI_SOCKTYPE, "ai_socktype not supported", 0 },
-  { EAI_SERVICE, "service not supported for ai_socktype", 0 },
-  { EAI_ADDRFAMILY, "address family for name not supported", 0 },
-  { EAI_MEMORY, "memory allocation failure", 0 },
-  { EAI_SYSTEM, "system error", 0 },
-  { 0, NULL, 0 }
+static struct nrl_nametonum socktypenametonum[] = {
+  { SOCK_RAW, "raw", 0 },
+  { SOCK_STREAM, "stream", 0 },
+  { SOCK_DGRAM, "dgram", 0 },
+#ifndef SOCK_UNSPEC
+#define SOCK_UNSPEC 0
+#endif /* SOCK_UNSPEC */
+  { SOCK_UNSPEC, "unspec", 0 },
+  { 0, NULL, -1 }
 };
 
 int nrl_nametonum(tab, name)
 struct nrl_nametonum *tab;
 char *name;
 {
-  for (; tab->name; tab++)
+  int i, j = -1;
+
+  i = strlen(name);
+
+  for (; tab->name; tab++) {
     if (!strcasecmp(tab->name, name))
       return(tab->num);
+    if (i && !strncmp(tab->name, name, i))
+      if (j < 0)
+        j = tab->num;
+      else {
+        i = 0; 
+        j = -1;
+      };
+  };
 
-  return -1;
+  return j;
 }
 
 char *nrl_numtoname(tab, num)
@@ -148,48 +164,43 @@ char *name;
   return i;
 }
 
-char *nrl_afnumtoname(num)
-int num;
+char *nrl_afnumtoname(int num)
 {
   char *c = nrl_numtoname(afnametonum, num);
  
   if (!c) {
-    buffer[0] = 'a';
-    buffer[1] = 'f';
-    buffer[2] = ' ';
+    strcpy(buffer, "af ");
     buffer[3] = hextab[(num & 0xf0) >> 4];
     buffer[4] = hextab[num & 0x0f];
     buffer[5] = 0;
-  }
+    return buffer;
+  };
 
   return c;
-}
+};
 
-char *nrl_gai_strerror(num)
-int num;
+int nrl_socktypenametonum(char *name)
 {
-  char *c = nrl_numtoname(gainametonum, num);
+  int i = nrl_nametonum(socktypenametonum, name);
+  if ((i < 0) && isdigit(*name))
+    i = atoi(name);
+  return i;
+};
+
+char *nrl_socktypenumtoname(int num)
+{
+  char *c = nrl_numtoname(socktypenametonum, num);
  
   if (!c) {
-    buffer[0] = '?';
-    buffer[1] = ' ';
-    buffer[2] = hextab[(num & 0xf0) >> 4];
-    buffer[3] = hextab[num & 0x0f];
-    buffer[4] = 0;
-  }
+    strcpy(buffer, "socktype ");    
+    buffer[9] = hextab[(num & 0xf0) >> 4];
+    buffer[10] = hextab[num & 0x0f];
+    buffer[11] = 0;
+    return buffer;
+  };
 
   return c;
-}
-
-#if INET6
-const struct in6_addr in6addr_any = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
-const struct in6_addr in6addr_loopback = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 };
-
-int inet6_isipv4mapped(const struct in6_addr *addr)
-{
-  return IN6_IS_ADDR_V4MAPPED(addr);
-}
-#endif /* INET6 */
+};
 
 static char *domain = NULL;
 static char domainbuffer[MAXHOSTNAMELEN] = "";
@@ -233,13 +244,15 @@ char *nrl_domainname(void)
 int nrl_build_samask(struct sockaddr *mask, int af, int len)
 {
   u_int8_t *p;
-  if (len < 0)
+  if (len < -1)
     return -1;
   switch(af) {
 #if INET6
     case AF_INET6:
       if (len > 128)
-      return -1;
+        return -1;
+      if (len == -1)
+        len = 128;
       p = (u_int8_t *)&(((struct sockaddr_in6 *)mask)->sin6_addr);
       memset(mask, 0, sizeof(struct sockaddr_in6));
 #if 0
@@ -249,13 +262,21 @@ int nrl_build_samask(struct sockaddr *mask, int af, int len)
 #endif /* INET6 */
     case AF_INET:
       if (len > 32)
-      return -1;
+        return -1;
+      if (len == -1)
+        len = 32;
       p = (u_int8_t *)&(((struct sockaddr_in *)mask)->sin_addr);
       memset(mask, 0, sizeof(struct sockaddr_in));
 #if 0
       ((struct sockaddr_in *)mask)->sin_port = ~1;
 #endif /* 0 */
       break;
+#if LOCAL
+    case AF_LOCAL:
+      len = 0;
+      memset(mask, 0, sizeof(struct sockaddr_in));
+      break;
+#endif /* LOCAL */
     default:
       return -1;
   };
@@ -271,6 +292,7 @@ int nrl_build_samask(struct sockaddr *mask, int af, int len)
     *p = (*p << 1) | 1;
   return 0;
 };
+
 int nrl_prefix_length(struct sockaddr *mask, int af)
 {
   int len = 0;
