@@ -340,7 +340,7 @@ setpeer(argc, argv)
           req.ai_socktype = SOCK_DGRAM;
 
           if (i = getaddrinfo(argv[1], service, &req, &ai)) {
-            printf("tftp: getaddrinfo: %s.%s: %s\n", argv[1], service, nrl_gai_strerror(i));
+            printf("tftp: getaddrinfo: %s.%s: %s\n", argv[1], service, gai_strerror(i));
             connected = 0;
             return;
           }
@@ -514,7 +514,7 @@ put(argc, argv)
 		  req.ai_socktype = SOCK_DGRAM;
 
 		  if (i = getaddrinfo(argv[1], service, &req, &ai)) {
-		    printf("tftp: getaddrinfo: %s.%s: %s\n", argv[1], service, nrl_gai_strerror(i));
+		    printf("tftp: getaddrinfo: %s.%s: %s\n", argv[1], service, gai_strerror(i));
 		    return;
 		  }
 
@@ -652,7 +652,7 @@ get(argc, argv)
 			  req.ai_socktype = SOCK_DGRAM;
 
 			  if (i = getaddrinfo(argv[1], service, &req, &ai)) {
-			    printf("tftp: getaddrinfo: %s.%s: %s\n", argv[1], service, nrl_gai_strerror(i));
+			    printf("tftp: getaddrinfo: %s.%s: %s\n", argv[1], service, gai_strerror(i));
 			    return;
 			  }
 
